@@ -9,12 +9,14 @@ module Network.Wai.Handler.Warp.Request (
   , headerLines
   , pauseTimeoutKey
   , getFileInfoKey
+#ifdef MIN_VERSION_x509
   , getClientCertificateKey
+#endif
   , NoKeepAliveRequest (..)
   ) where
 
 import qualified Control.Concurrent as Conc (yield)
-import Control.Exception (throwIO, Exception)
+import UnliftIO (throwIO, Exception)
 import Data.Array ((!))
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Unsafe as SU
@@ -22,7 +24,9 @@ import qualified Data.CaseInsensitive as CI
 import qualified Data.IORef as I
 import Data.Typeable (Typeable)
 import qualified Data.Vault.Lazy as Vault
+#ifdef MIN_VERSION_x509
 import Data.X509
+#endif
 import qualified Network.HTTP.Types as H
 import Network.Socket (SockAddr)
 import Network.Wai
@@ -72,7 +76,9 @@ recvRequest firstRequest settings conn ii th addr src transport = do
         rawPath = if settingsNoParsePath settings then unparsedPath else path
         vaultValue = Vault.insert pauseTimeoutKey (Timeout.pause th)
                    $ Vault.insert getFileInfoKey (getFileInfo ii)
+#ifdef MIN_VERSION_x509
                    $ Vault.insert getClientCertificateKey (getTransportClientCertificate transport)
+#endif
                      Vault.empty
     (rbody, remainingRef, bodyLength) <- bodyAndSource src cl te
     -- body producing function which will produce '100-continue', if needed
@@ -110,7 +116,7 @@ headerLines maxTotalHeaderLength firstRequest src = do
         -- lack of data as a real exception. See the http1 function in
         -- the Run module for more details.
         then if firstRequest then throwIO ConnectionClosedByPeer else throwIO NoKeepAliveRequest
-        else push maxTotalHeaderLength src (THStatus 0 id id) bs
+        else push maxTotalHeaderLength src (THStatus 0 0 id id) bs
 
 data NoKeepAliveRequest = NoKeepAliveRequest
     deriving (Show, Typeable)
@@ -209,7 +215,8 @@ type BSEndo = ByteString -> ByteString
 type BSEndoList = [ByteString] -> [ByteString]
 
 data THStatus = THStatus
-    {-# UNPACK #-} !Int -- running total byte count
+    !Int -- running total byte count (excluding current header chunk)
+    !Int -- current header chunk byte count
     BSEndoList -- previously parsed lines
     BSEndo -- bytestrings to be prepended
 
@@ -221,29 +228,41 @@ close = throwIO IncompleteHeaders
 -}
 
 push :: Int -> Source -> THStatus -> ByteString -> IO [ByteString]
-push maxTotalHeaderLength src (THStatus len lines prepend) bs'
+push maxTotalHeaderLength src (THStatus totalLen chunkLen lines prepend) bs'
         -- Too many bytes
-        | len > maxTotalHeaderLength = throwIO OverLargeHeader
-        | otherwise = push' mnl
+        | currentTotal > maxTotalHeaderLength = throwIO OverLargeHeader
+        | otherwise = push' mNL
   where
+    currentTotal = totalLen + chunkLen
+    -- bs: current header chunk, plus maybe (parts of) next header
     bs = prepend bs'
     bsLen = S.length bs
-    mnl = do
-        nl <- S.elemIndex 10 bs
+    -- Maybe newline
+    -- Returns: Maybe
+    --    ( length of this chunk up to newline
+    --    , position of newline in relation to entire current header
+    --    , is this part of a multiline header
+    --    )
+    mNL = do
+        -- 10 is the code point for newline (\n)
+        chunkNL <- S.elemIndex 10 bs'
+        let headerNL = chunkNL + S.length (prepend "")
+            chunkNLlen = chunkNL + 1
         -- check if there are two more bytes in the bs
         -- if so, see if the second of those is a horizontal space
-        if bsLen > nl + 1 then
-            let c = S.index bs (nl + 1)
-                b = case nl of
+        if bsLen > headerNL + 1 then
+            let c = S.index bs (headerNL + 1)
+                b = case headerNL of
                       0 -> True
                       1 -> S.index bs 0 == 13
                       _ -> False
-            in Just (nl, not b && (c == 32 || c == 9))
+                isMultiline = not b && (c == 32 || c == 9)
+            in Just (chunkNLlen, headerNL, isMultiline)
             else
-            Just (nl, False)
+            Just (chunkNLlen, headerNL, False)
 
     {-# INLINE push' #-}
-    push' :: Maybe (Int, Bool) -> IO [ByteString]
+    push' :: Maybe (Int, Int, Bool) -> IO [ByteString]
     -- No newline find in this chunk.  Add it to the prepend,
     -- update the length, and continue processing.
     push' Nothing = do
@@ -251,26 +270,32 @@ push maxTotalHeaderLength src (THStatus len lines prepend) bs'
         when (S.null bst) $ throwIO IncompleteHeaders
         push maxTotalHeaderLength src status bst
       where
-        len' = len + bsLen
         prepend' = S.append bs
-        status = THStatus len' lines prepend'
+        thisChunkLen = S.length bs'
+        newChunkLen = chunkLen + thisChunkLen
+        status = THStatus totalLen newChunkLen lines prepend'
     -- Found a newline, but next line continues as a multiline header
-    push' (Just (end, True)) = push maxTotalHeaderLength src status rest
+    push' (Just (chunkNLlen, end, True)) =
+        push maxTotalHeaderLength src status rest
       where
         rest = S.drop (end + 1) bs
         prepend' = S.append (SU.unsafeTake (checkCR bs end) bs)
-        len' = len + end
-        status = THStatus len' lines prepend'
+        -- If we'd just update the entire current chunk up to newline
+        -- we wouldn't count all the dropped newlines in between.
+        -- So update 'chunkLen' with current chunk up to newline
+        -- and use 'chunkLen' later on to add to 'totalLen'.
+        newChunkLen = chunkLen + chunkNLlen
+        status = THStatus totalLen newChunkLen lines prepend'
     -- Found a newline at position end.
-    push' (Just (end, False))
+    push' (Just (chunkNLlen, end, False))
       -- leftover
       | S.null line = do
             when (start < bsLen) $ leftoverSource src (SU.unsafeDrop start bs)
             return (lines [])
       -- more headers
-      | otherwise   = let len' = len + start
-                          lines' = lines . (line:)
-                          status = THStatus len' lines' id
+      | otherwise   = let lines' = lines . (line:)
+                          newTotalLength = totalLen + chunkLen + chunkNLlen
+                          status = THStatus newTotalLength 0 lines' id
                       in if start < bsLen then
                              -- more bytes in this chunk, push again
                              let bs'' = SU.unsafeDrop start bs
@@ -286,7 +311,7 @@ push maxTotalHeaderLength src (THStatus len lines prepend) bs'
 
 {-# INLINE checkCR #-}
 checkCR :: ByteString -> Int -> Int
-checkCR bs pos = if pos > 0 && 13 == S.index bs p then p else pos -- 13 is CR
+checkCR bs pos = if pos > 0 && 13 == S.index bs p then p else pos -- 13 is CR (\r)
   where
     !p = pos - 1
 
@@ -298,6 +323,8 @@ getFileInfoKey :: Vault.Key (FilePath -> IO FileInfo)
 getFileInfoKey = unsafePerformIO Vault.newKey
 {-# NOINLINE getFileInfoKey #-}
 
+#ifdef MIN_VERSION_x509
 getClientCertificateKey :: Vault.Key (Maybe CertificateChain)
 getClientCertificateKey = unsafePerformIO Vault.newKey
 {-# NOINLINE getClientCertificateKey #-}
+#endif
