@@ -15,7 +15,7 @@ module Network.Wai.Handler.Warp.Response (
   ) where
 
 import Data.ByteString.Builder.HTTP.Chunked (chunkedTransferEncoding, chunkedTransferTerminator)
-import qualified Control.Exception as E
+import qualified UnliftIO
 import Data.Array ((!))
 import qualified Data.ByteString as S
 import Data.ByteString.Builder (byteString, Builder)
@@ -275,9 +275,9 @@ sendRsp conn ii th ver s0 hs0 rspidxhdr (RspFile path (Just part) _ isHead hook)
 -- Simple WAI applications.
 -- Status is ignored
 sendRsp conn ii th ver _ hs0 rspidxhdr (RspFile path Nothing reqidxhdr isHead hook) = do
-    efinfo <- E.try $ getFileInfo ii path
+    efinfo <- UnliftIO.tryIO $ getFileInfo ii path
     case efinfo of
-        Left (_ex :: E.IOException) ->
+        Left (_ex :: UnliftIO.IOException) ->
 #ifdef WARP_DEBUG
           print _ex >>
 #endif
