@@ -4,11 +4,13 @@
 
 module Network.Wai.Handler.Warp.Types where
 
-import Control.Exception
+import qualified UnliftIO
 import qualified Data.ByteString as S
 import Data.IORef (IORef, readIORef, writeIORef, newIORef)
 import Data.Typeable (Typeable)
+#ifdef MIN_VERSION_x509
 import Data.X509
+#endif
 import Foreign.Ptr (Ptr)
 import System.Posix.Types (Fd)
 import qualified System.TimeManager as T
@@ -49,7 +51,7 @@ instance Show InvalidRequest where
     show OverLargeHeader = "Warp: Request headers too large, possible memory attack detected. Closing connection."
     show (BadProxyHeader s) = "Warp: Invalid PROXY protocol header: " ++ show s
 
-instance Exception InvalidRequest
+instance UnliftIO.Exception InvalidRequest
 
 ----------------------------------------------------------------
 
@@ -60,10 +62,10 @@ instance Exception InvalidRequest
 -- Used to determine whether keeping the HTTP1.1 connection / HTTP2 stream alive is safe
 -- or irrecoverable.
 
-newtype ExceptionInsideResponseBody = ExceptionInsideResponseBody SomeException
+newtype ExceptionInsideResponseBody = ExceptionInsideResponseBody UnliftIO.SomeException
     deriving (Show, Typeable)
 
-instance Exception ExceptionInsideResponseBody
+instance UnliftIO.Exception ExceptionInsideResponseBody
 
 ----------------------------------------------------------------
 
@@ -180,12 +182,16 @@ data Transport = TCP -- ^ Plain channel: TCP
                  , tlsMinorVersion :: Int
                  , tlsNegotiatedProtocol :: Maybe ByteString -- ^ The result of Application Layer Protocol Negociation in RFC 7301
                  , tlsChiperID :: Word16
+#ifdef MIN_VERSION_x509
                  , tlsClientCertificate :: Maybe CertificateChain
+#endif
                  }  -- ^ Encrypted channel: TLS or SSL
                | QUIC {
                    quicNegotiatedProtocol :: Maybe ByteString
                  , quicChiperID :: Word16
+#ifdef MIN_VERSION_x509
                  , quicClientCertificate :: Maybe CertificateChain
+#endif
                  }
 
 isTransportSecure :: Transport -> Bool
@@ -196,7 +202,9 @@ isTransportQUIC :: Transport -> Bool
 isTransportQUIC QUIC{} = True
 isTransportQUIC _      = False
 
+#ifdef MIN_VERSION_x509
 getTransportClientCertificate :: Transport -> Maybe CertificateChain
 getTransportClientCertificate TCP              = Nothing
 getTransportClientCertificate (TLS _ _ _ _ cc) = cc
 getTransportClientCertificate (QUIC _ _ cc)    = cc
+#endif
