@@ -6,8 +6,8 @@ module RunSpec (main, spec, withApp, MySocket, msWrite, msRead, withMySocket) wh
 import Control.Concurrent (forkIO, killThread, threadDelay)
 import Control.Concurrent.MVar (newEmptyMVar, takeMVar, putMVar)
 import Control.Concurrent.STM
-import qualified Control.Exception as E
-import Control.Exception.Lifted (bracket, try, IOException, onException)
+import qualified UnliftIO.Exception as E
+import UnliftIO.Exception (bracket, try, IOException, onException)
 import Control.Monad (forM_, replicateM_, unless)
 import Control.Monad.IO.Class (MonadIO, liftIO)
 import Data.ByteString (ByteString)
